#include <cmath>
#include <MusicModel.hpp>
#include <random>
#include <vector>

int main(int argc, const char **argv)
{
    csound::MusicModel model;
    model.setTitle("\"Example1\"");
    model.setFilename("ComposingInC++/Example1");
    model.setAlbum("Silence");
    model.setArtist("\"Michael Gogins\"");
    model.setCopyright("\"(C) 2012 by Michael Gogins\"");
    
    csound::Score &score = model.getScore();
    double interval = 0.25;
    double voices = 3;
    double scoreduration = 180;
    double duration = voices * interval;
    double velocity = 84.3090 - (voices * 6);
    double c = .9859730849;
    double y = 0.5;
    double y1 = 0.5;
    std::mt19937 mersenneTwister;
    std::uniform_real_distribution<> randomvariable(-0.9, +0.9);
    for (double time_  = 0.5; time_ < scoreduration; time_ += interval) {
        y1 = c * y * (1.0 - y) * 4.0;
        y = y1;
        double status = 144.0;
        double insno = 1.0;
        double key = std::floor(24. + y * 84.);
        double phase = 0.0;
        double pan = randomvariable(mersenneTwister);
        score.append(time_, duration, status, insno, key, velocity, phase, pan);
    }
    score.tieOverlappingNotes();

    model.setCsoundOrchestra(R"(

sr                              =                       48000
ksmps                           =                       128
nchnls                          =                       2
0dbfs					        =                      	1000000
gSnapshotfile                   =                       "Example1.snap"
iampdbfs                        init                    32768
                                prints                  "Default amplitude at 0 dBFS:  %9.4f\n", iampdbfs
idbafs                          init                    dbamp(iampdbfs)
                                prints                  "dbA at 0 dBFS:                 %9.4f\n", idbafs
iheadroom                       init                    6
                                prints                  "Headroom (dB):                 %9.4f\n", iheadroom
idbaheadroom                    init                    idbafs - iheadroom
                                prints                  "dbA at headroom:               %9.4f\n", idbaheadroom
iampheadroom                    init                    ampdb(idbaheadroom)
                                prints                  "Amplitude at headroom:        %9.4f\n", iampheadroom
                                prints                  "Balance so the overall amps at the end of performance -6 dbfs.\n"

giSliderStyle                   =                       5
ivaluatorwidth                  =                       120
iwidgetheight                   =                       20
idisplaywidth                   =                       80
iguibox                         =                       2
iguioffset                      =                       3
iwidgetoffset                   =                       16
ipanelwidth                     =                       600
ipanelheight                    =                       500
                                FLpanel                 "Csound", ipanelwidth, ipanelheight, 10, 10, iguibox, 0, 1
itabswidth                      =                       ipanelwidth - (2 * iguioffset)
itabsheight                     =                       ipanelheight - (2 * iguioffset)
                                FLtabs                  itabswidth, itabsheight, iguioffset, iguioffset
itabwidth                       =                       itabswidth
itabtabheight                   =                       iwidgetheight + 2 * iguioffset
itabheight                      =                       itabsheight - itabtabheight
ifirstwidgetx                   =                       iwidgetoffset + 2 * iguioffset
ifirstwidgety                   =                       iwidgetoffset + itabtabheight + 2 * iguioffset
                                
giFlatQ                         init                    sqrt(0.5)
giseed				            init                    0.5

                                connect                 "Compressor",           "outleft", 	"ParEq1",           "inleft"
                                connect                 "Compressor",           "outright", "ParEq1",           "inright"
                                connect                 "ModerateFM",          	"outleft", 	"Reverb",           "inleft"
                                connect                 "ModerateFM",          	"outright", "Reverb",           "inright"
                                connect                 "ParEq1",               "outleft", 	"ParEq2",           "inleft"
                                connect                 "ParEq1",               "outright", "ParEq2",           "inright"
                                connect                 "ParEq2",               "outleft", 	"MasterOutput",     "inleft"
                                connect                 "ParEq2",               "outright", "MasterOutput",     "inright"
                                connect                 "Reverb",               "outleft", 	"Compressor",       "inleft"
                                connect                 "Reverb",               "outright", "Compressor",       "inright"
                                
                                alwayson                "Reverb"
                                alwayson                "Compressor"
                                alwayson                "ParEq1"
                                alwayson                "ParEq2"
                                alwayson                "MasterOutput"
                                
                                //////////////////////////////////////////////
                                // ModerateFM
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "ModerateFM", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset , iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety  
gkModerateFMCarrier, giModerateFMCarrier    FLslider    "Carrier", 0.5, 2, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1.0, giModerateFMCarrier
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkModerateFMModulator, giModerateFMModulator    FLslider "Modulator", 0.0, 10.0, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              0.5, giModerateFMModulator
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkModerateFMIndex, giModerateFMIndex    FLslider        "Index", 0.0, 10.0, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              0.25, giModerateFMIndex
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkModerateFMAmplitude, giModerateFMAmplitude    FLslider "FM amplitude", 0.0, 10.0, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              0.25, giModerateFMAmplitude
                                FLgroupEnd
                                instr			        ModerateFM
                                pset                    0, 0, 1000
i_instrument                    =                       p1
i_time                          =                       p2
i_duration                      =                       p3
i_midikey                       =                       p4
i_midivelocity                  =                       p5
i_phase                         =                       p6
i_pan                           =                       p7
i_depth                         =                       p8
i_height                        =                       p9
i_pitchclassset                 =                       p10
i_homogeneity                   =                       p11
iattack			                =			            0.002
isustain		                =			            p3
idecay				            =			            8
irelease		                =			            0.05
iHz                             =                       cpsmidinn(i_midikey)
idB                             =                       i_midivelocity
iamplitude                      =                       ampdb(idB) * 4.0
ifrequencyb             	    =                       iHz * 1.003
kcarrierb               	    =                       gkModerateFMCarrier * 1.004
aindenv                 	    transeg                 0.0, iattack, -11.0, 1.0, idecay, -7.0, 0.025, isustain, 0.0, 0.025, irelease, -17.0, 0.0
aindex                  	    =                       aindenv * gkModerateFMIndex * gkModerateFMAmplitude
kindex                          downsamp                aindex, ksmps
isinetable                      ftgenonce               0, 0, 65536, 10, 1, 0, .02
aouta                   	    foscili                 1.0, iHz, gkModerateFMCarrier, gkModerateFMModulator, kindex / 4., isinetable
aoutb                   	    foscili                 1.0, ifrequencyb, kcarrierb, gkModerateFMModulator, kindex, isinetable
asignal               		    =                       (aouta + aoutb) * aindenv
adeclick                        linsegr                 0, iattack, 1, isustain, 1, irelease, 0
asignal                         =                       asignal * iamplitude
aoutleft, aoutright             pan2                    asignal * adeclick, i_pan
                                outleta                 "outleft",  aoutleft
                                outleta                 "outright", aoutright
                                prints                  "instr %4d t %9.4f d %9.4f k %9.4f v %9.4f p %9.4f\n", p1, p2, p3, p4, p5, p7
                                endin

                                //////////////////////////////////////////////
                                // OUTPUT INSTRUMENTS MUST GO BELOW HERE
                                //////////////////////////////////////////////

                                //////////////////////////////////////////////
                                // Reverb
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "Reverb", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset, iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety  
gkReverbEnabled, giReverbEnabled FLbutton               "Enabled", 1, 0, 3, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              0, giReverbEnabled
iy                              =                       iy + 2 * iwidgetheight + iguioffset

gkReverbWet, giReverbWet        FLslider                "Wet", 0, 1, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giReverbWet
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkReverbDelay, giReverbDelay    FLslider                "Delay", 0, 1, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giReverbDelay
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkReverbCutoff, giReverbCutoff  FLslider                "Cutoff", 20, 20000, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giReverbCutoff
                                FLgroupEnd
                                instr                   Reverb
ainleft                         inleta                  "inleft"
ainright                        inleta                  "inright"
if (gkReverbEnabled == 0) then
aoutleft                        =                       ainleft
aoutright                       =                       ainright
kdry				            =			            1.0 - gkReverbWet
else
awetleft, awetright             reverbsc                ainleft, ainright, gkReverbDelay, gkReverbCutoff
aoutleft			            =			            ainleft *  kdry + awetleft  * gkReverbWet
aoutright			            =			            ainright * kdry + awetright * gkReverbWet
endif
                                outleta                 "outleft", aoutleft
                                outleta                 "outright", aoutright
                                endin

                                //////////////////////////////////////////////
                                // Compressor
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "Compressor", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset , iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety                                 
gkCompressorEnabled, giCompressorEnabled    FLbutton                "Enabled", 1, 0, 3, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              0, giCompressorEnabled
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkCompressorThreshold, giCompressorThreshold  FLslider  "Threshhold", 0.000001, 1, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorThreshold
iy                              =                       iy + 2 * iwidgetheight + iguioffset

gkCompressorLowKnee, giCompressorLowKnee  FLslider      "Low knee", 0.000001, 100, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorLowKnee
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkCompressorHighKnee, giCompressorHighKnee  FLslider    "High knee", 0.000001, 100, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorHighKnee
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkCompressorRatio, giCompressorRatio  FLslider          "Ratio", 0.000001, 100, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorRatio
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkCompressorAttack, giCompressorAttack  FLslider        "Attack", 0.000001, 0.1, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorAttack
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkCompressorRelease, giCompressorRelease  FLslider      "Release", 0.000001, 0.1, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giCompressorRelease
                                FLgroupEnd
                                instr                   Compressor
ainleft                         inleta                  "inleft"
ainright                        inleta                  "inright"
                                if (gkCompressorEnabled == 0) then
aoutleft                        =                       ainleft
aoutright                       =                       ainright
                                else
aoutleft                        compress                ainleft,        ainleft,  gkCompressorThreshold, gkCompressorLowKnee, gkCompressorHighKnee, gkCompressorRatio, gkCompressorAttack, gkCompressorRelease, .05
aoutright                       compress                ainright,       ainright, gkCompressorThreshold, gkCompressorLowKnee, gkCompressorHighKnee, gkCompressorRatio, gkCompressorAttack, gkCompressorRelease, .05
                                endif
                                outleta                 "outleft",      aoutleft
                                outleta                 "outright",     aoutright
                                endin   

                                //////////////////////////////////////////////
                                // ParEq1
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "EQ 1", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset , iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety  
gkParEq1Enabled, giParEq1Enabled    FLbutton             "Enabled", 1, 0, 3, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              0, giParEq1Enabled
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq1Mode, giParEq1Mode      FLcount                 "Mode", 0, 2, 1, 0, 2, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              1, giParEq1Mode
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq1Frequency, giParEq1Frequency  FLslider          "Frequency", 20, 20000, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giParEq1Frequency
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq1Gain, giParEq1Gain      FLslider                "Gain", -20, 20, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              0, giParEq1Gain
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq1Q, giParEq1Q            FLslider                "Q", 0, 10, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giParEq1Q
                                FLgroupEnd
                                instr                   ParEq1
ainleft                         inleta                  "inleft"
ainright                        inleta                  "inright"
                                if (gkParEq1Enabled == 0) then
aoutleft                        =                       ainleft
aoutright                       =                       ainright
                                else
kamplitude                      =                       ampdb(gkParEq1Gain)
                                if      (gkParEq1Mode == 0) then
aoutleft                        pareq                   ainleft,        gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 0
aoutright                       pareq                   ainright,       gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 0
                                elseif  (gkParEq1Mode == 1) then
aoutleft                        pareq                   ainleft,        gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 1
aoutright                       pareq                   ainright,       gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 1
                                elseif  (gkParEq1Mode == 2) then
aoutleft                        pareq                   ainleft,        gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 2
aoutright                       pareq                   ainright,       gkParEq1Frequency, kamplitude, giFlatQ + gkParEq1Q, 2
                                endif
                                endif
                                outleta                 "outleft",  aoutleft
                                outleta                 "outright", aoutright
                                endin

                                //////////////////////////////////////////////
                                // ParEq2
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "EQ 2", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset , iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety  
gkParEq2Enabled, giParEq2Enabled    FLbutton             "Enabled", 1, 0, 3, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              0, giParEq2Enabled
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq2Mode, giParEq2Mode      FLcount                 "Mode", 0, 2, 1, 0, 2, ivaluatorwidth, iwidgetheight, ix, iy, -1
                                FLsetVal_i              1, giParEq2Mode
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq2Frequency, giParEq2Frequency  FLslider          "Frequency", 20, 20000, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giParEq2Frequency
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq2Gain, giParEq2Gain      FLslider                "Gain", -20, 20, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              0, giParEq2Gain
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkParEq2Q, giParEq2Q            FLslider                "Q", 0, 10, 0, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giParEq2Q
                                FLgroupEnd
                                instr                   ParEq2
ainleft                         inleta                  "inleft"
ainright                        inleta                  "inright"
                                if                      (gkParEq2Enabled == 0) then
aoutleft                        =                       ainleft
aoutright                       =                       ainright
                                else
                                if                      (gkParEq2Mode == 0) then
aoutleft                        pareq                   ainleft, 	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 0
aoutright                       pareq                   ainright,	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 0
                                elseif                  (gkParEq2Mode == 1) then
aoutleft                        pareq                   ainleft, 	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 1
aoutright                       pareq                   ainright,	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 1
                                elseif                  (gkParEq2Mode == 2) then
aoutleft                        pareq                   ainleft, 	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 2
aoutright                       pareq                   ainright,	gkParEq2Frequency, gkParEq2Gain, giFlatQ + gkParEq2Q, 2
                                endif
                                endif
                                outleta                 "outleft", 	aoutleft
                                outleta                 "outright", aoutright
                                endin

                                //////////////////////////////////////////////
                                // MasterOutput
                                // By Michael Gogins.
                                //////////////////////////////////////////////
                                FLgroup                 "Master", itabwidth, itabheight, iguioffset, itabtabheight + iguioffset , iguibox
                                ix = ifirstwidgetx
                                iy = ifirstwidgety  
gkMasterLevel, giMasterLevel    FLslider                "Output level", 0.0001, 1000, -1, giSliderStyle, -1, ivaluatorwidth, iwidgetheight, ix, iy
                                FLsetVal_i              1, giMasterLevel
iy                              =                       iy + 2 * iwidgetheight + iguioffset
                                FLsetSnapGroup          1
gkSnapSave, giSnapSave          FLbutton                "Save snapshot", 1, 0, 1, ivaluatorwidth, iwidgetheight, ix, iy, 0, 200, 0, 3
iy                              =                       iy + 2 * iwidgetheight + iguioffset
gkSnapLoad, giSnapLoad          FLbutton                "Load snapshot", 1, 0, 1, ivaluatorwidth, iwidgetheight, ix, iy, 0, 201, 0, 3
                                FLgroupEnd
                                
                                instr                   MasterOutput
ivalid                          filevalid               gSnapshotfile
                                if ivalid == 1          then
                                prints                  "Loading..."
                                FLloadsnap              gSnapshotfile 
ivaluecount                     FLgetsnap               0
                                prints                  "%d values set.\n", ivaluecount
                                else
                                prints                  "Missing or invalid snapshot file."
                                endif
ainleft                         inleta                  "inleft"
ainright                        inleta                  "inright"
aoutleft                        =                       gkMasterLevel * ainleft
aoutright                       =                       gkMasterLevel * ainright
                                outs                    aoutleft, aoutright
                                endin
                                                                
                                instr                   200 
                                prints                  "Saving..."
isnapshot, ivaluecount          FLsetsnap               0
                                FLsavesnap              gSnapshotfile
                                endin

                                instr                   201 
ivalid                          filevalid               gSnapshotfile
                                if ivalid == 1          then
                                prints                  "Loading..."
                                FLloadsnap              gSnapshotfile 
ivaluecount                     FLgetsnap               0
                                else
                                prints                  "Missing or invalid snapshot file."
                                endif
                                endin
                                
                                FLtabsEnd
                                FLpanelEnd
                                FLrun
                                                                
            )");

    model.processArgv(argc, argv);
}

